@@ -1,6 +1,6 @@
 from .access import AccessControl
 from .account_settings import AccountSettings
-from .aes_encrypt import AESEncrypt
+from .aes_encrypt import AESEncrypt, AESEncryptBin
 from .basic_crud import BasicCRUD
 from .exceptions import *  # noqa
 from .grants import Grants
@@ -11,6 +11,6 @@ from .service_registry import ServiceRegistry
 from .session_auth import SessionAuth
 from .session_manager import SessionManager, Mutex
 
-__all__ = ('AccessControl', 'AccountSettings', 'AESEncrypt', 'BasicCRUD',
-           'Grants', 'Metrics', 'Mutex', 'RateLimit', 'ServiceConfig',
-           'ServiceRegistry', 'SessionAuth', 'SessionManager')
+__all__ = ('AccessControl', 'AccountSettings', 'AESEncrypt', 'AESEncryptBin',
+           'BasicCRUD', 'Grants', 'Metrics', 'Mutex', 'RateLimit',
+           'ServiceConfig', 'ServiceRegistry', 'SessionAuth', 'SessionManager')
