@@ -2,7 +2,7 @@
 
 created 26-mar-2021 by richb@instantlinux.net
 """
-from flask import g, session as flask_session
+from flask import g
 from flask_babel import _
 import logging
 from sqlalchemy.orm.exc import NoResultFound
@@ -11,7 +11,7 @@ import string
 from .. import state
 from ..database import db_abort
 from ..metrics import Metrics
-from ..session_auth import Ocache, SessionAuth
+from ..session_auth import SessionAuth
 from ..utils import gen_id, identity_normalize
 
 
@@ -39,9 +39,6 @@ class OAuth2(SessionAuth):
             msg = _(u'login client missing')
             logging.error(dict(message=msg, **logmsg))
             return dict(message=msg), 405
-        if self.config.AUTH_SKIP_CORS:
-            flask_session['_%s_authlib_state_' % method] = Ocache().get(
-                'session_foo_state')
         try:
             token = client.authorize_access_token(
                 redirect_uri='%s%s/%s/%s' % (self.config.PUBLIC_URL,
@@ -49,8 +46,7 @@ class OAuth2(SessionAuth):
                                              'auth_callback', method))
         except Exception as ex:
             msg = _(u'openid client failure')
-            logging.warning(dict(message=msg, error=str(ex),
-                                 suggest='for dev: AUTH_SKIP_CORS', **logmsg))
+            logging.warning(dict(message=msg, error=str(ex), **logmsg))
             return dict(message=msg), 405
         if 'id_token' in token:
             user = client.parse_id_token(token)
@@ -63,29 +59,38 @@ class OAuth2(SessionAuth):
             logging.warning(dict(message='identity missing', usermeta=user,
                                  **logmsg))
             return dict(message=_(u'access denied')), 403
+        try:
+            contact = g.db.query(self.models.Contact).filter_by(
+                    info=identity).one()
+        except NoResultFound:
+            return self._handle_unknown_user(method, user)
+        except Exception as ex:
+            return db_abort(str(ex), **logmsg)
         try:
             account = g.db.query(self.models.Account).join(
                 self.models.Person).filter(
                     self.models.Person.identity == identity,
                     self.models.Account.status == 'active').one()
-            username = account.name
         except NoResultFound:
-            account = None
+            pass
         except Exception as ex:
             return db_abort(str(ex), **logmsg)
         if not account:
             try:
                 account = g.db.query(self.models.Account).join(
                     self.models.Person).join(self.models.Contact).filter(
-                    self.models.Contact.info == identity,
-                    self.models.Contact.type == 'email',
-                    self.models.Contact.status == 'active',
-                    self.models.Account.status == 'active').one()
-                username = account.name
+                        self.models.Contact.info == identity,
+                        self.models.Contact.type == 'email').one()
             except NoResultFound:
-                return self._handle_unknown_user(method, user)
-        logging.info(dict(usermeta=user, **logmsg))
-        return self.login_accepted(username, account, method)
+                logging.warning(dict(message='lookup failed', **logmsg))
+                return dict(message=_(u'access denied')), 403
+            except Exception as ex:
+                return db_abort(str(ex), **logmsg)
+        if (account.status != 'active' or account.owner.status != 'active'
+                or contact.status == 'inactive'):
+            logging.info(dict(message='disabled account', **logmsg))
+            return dict(message=_(u'access denied')), 403
+        return self.login_accepted(account.name, account, method)
 
     def _handle_unknown_user(self, method, usermeta):
         """Handle unknown external user access based on configured
@@ -105,7 +110,7 @@ class OAuth2(SessionAuth):
             identity.split('@')[0][:15],
             gen_id(length=6, prefix='',
                    chars=string.digits + string.ascii_lowercase))
-        logmsg = dict(action='login', method=method, identity=identity)
+        logmsg = dict(action='register', method=method, identity=identity)
 
         if self.config.LOGIN_EXTERNAL_POLICY == 'closed':
             msg = _(u'not valid')
@@ -113,30 +118,23 @@ class OAuth2(SessionAuth):
             Metrics().store('logins_fail_total')
             return dict(message=msg), 403
         elif self.config.LOGIN_EXTERNAL_POLICY == 'auto':
-            # suppress the registration confirmation email
-            # TODO fix this and make this configurable
-            self.func_send = None
-            ret = self.register(identity, username, name, picture=picture)
+            ret = self.register(identity, username, name, picture=picture,
+                                template=None)
             if ret[1] != 200:
                 return ret
-            try:
-                g.db.query(self.models.Contact).filter_by(
-                    id=ret[0]['id']).one().status = 'active'
-                g.db.commit()
-            except Exception as ex:
-                return db_abort(str(ex), rollback=True,
-                                msg=_(u'contact activation problem'), **logmsg)
             ret = self.account_add(username, uid=ret[0]['uid'])
             if ret[1] != 201:
                 return ret
             account = g.db.query(self.models.Account).filter_by(
                 id=ret[0]['id']).one()
+            logging.info(dict(usermeta=usermeta, **logmsg))
             return self.login_accepted(username, account, method)
         elif self.config.LOGIN_EXTERNAL_POLICY == 'open':
             ret = self.register(identity, username, name, picture=picture)
             if ret[1] != 200:
                 return ret
             ret2 = self.account_add(username, uid=ret[0]['uid'])
+            logging.info(dict(usermeta=usermeta, **logmsg))
             return ret2 if ret2 == 201 else ret
 
         # TODO LOGIN_EXTERNAL_POLICY == 'onrequest'
